@@ -165,6 +165,7 @@ class Parser(ABC):
         allow_population_by_field_name: bool = False,
         apply_default_values_for_required_fields: bool = False,
         force_optional_for_required_fields: bool = False,
+        class_name: Optional[str] = None,
     ):
         self.data_type_manager: DataTypeManager = data_type_manager_type(
             target_python_version
@@ -210,6 +211,7 @@ class Parser(ABC):
             self.field_preprocessors.append(snakify_field)
         if self.strip_default_none:
             self.field_preprocessors.append(set_strip_default_none)
+        self.class_name: Optional[str] = class_name
 
     @property
     def iter_source(self) -> Iterator[Source]:
@@ -291,7 +293,14 @@ class Parser(ABC):
             for model in models:
                 alias_map: Dict[str, Optional[str]] = {}
                 if model.name in require_update_action_models:
-                    models_to_update += [model.name]
+                    ref_names = [
+                        d.reference.name
+                        for f in model.fields
+                        for d in f.data_type.all_data_types
+                        if d.reference
+                    ]
+                    if model.name in ref_names:
+                        models_to_update += [model.name]
                 imports.append(model.imports)
                 for field in model.fields:
                     for data_type in field.data_type.all_data_types:  # type: DataType
@@ -311,11 +320,17 @@ class Parser(ABC):
                             reference = self.model_resolver.get(
                                 data_type.reference.path
                             )
-                            if (
-                                reference
-                                and reference.actual_module_name == module_path
+                            if reference and (
+                                (
+                                    isinstance(self.source, Path)
+                                    and self.source.is_file()
+                                    and self.source.name
+                                    == reference.path.split('#/')[0]
+                                )
+                                or reference.actual_module_name == module_path
                             ):
-                                model.reference_classes.remove(name)
+                                if name in model.reference_classes:  # pragma: no cover
+                                    model.reference_classes.remove(name)
                                 continue
                         if full_path in alias_map:
                             alias = alias_map[full_path] or import_
@@ -354,9 +369,7 @@ class Parser(ABC):
 
             body = '\n'.join(result)
             if format_:
-                body = format_code(
-                    body, self.target_python_version, settings_path or Path().resolve()
-                )
+                body = format_code(body, self.target_python_version, settings_path)
 
             results[module] = Result(body=body, source=models[0].path)
 
